// --------------------------------------------------------------------
// --------------------------------------------------------------------
// OneWire DS18B20, DS1822 Temperature Example
// --------------------------------------------------------------------
// --------------------------------------------------------------------
#include <LiquidCrystal.h>
#include <OneWire.h>
// --------------------------------------------------------------------
LiquidCrystal lcd(P1_0, P1_1, P1_2, P1_3, P1_4, P1_5);
OneWire       ds18b20_1(11);  // on pin 11 = P2.3 (a 4.7K resistor is necessary)
OneWire       ds18b20_2(12);  // on pin 12 = P2.4 (a 4.7K resistor is necessary)
// --------------------------------------------------------------------
byte ROM_ID1[8];
byte ROM_ID2[8];
byte data1[12];
byte data2[12];
byte i;
int16_t t1_raw;
int16_t t2_raw;
float t1_in;
float t2_out;
byte v_data1_ok;
byte v_data2_ok;
byte v_index;
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void setup()
{
  lcd.begin(16, 2);
  lcd.setCursor(0, 0);
  lcd.print("Duo-Thermometer");
  lcd.setCursor(0, 1);
  lcd.print("2.03/04.01.2021");

  ds18b20_1.search(ROM_ID1);
  ds18b20_2.search(ROM_ID2);
  meassure_and_read_X(0);
  convert_t1();
  convert_t2();

  delay(2000);
  lcd.clear();
  v_index = 0;
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void loop()
{
  t1_in  = (float)t1_raw / 16.0;
  t2_out = (float)t2_raw / 16.0;
  
  lcd_print_temp_duo();

  ds18b20_1.search(ROM_ID1);
  ds18b20_2.search(ROM_ID2);
  meassure_and_read_X(1);
  convert_t1();
  convert_t2();
}
// --------------------------------------------------------------
// --------------------------------------------------------------
void convert_t1()
{
  v_data1_ok = 0x00;
  if ( data1[7] == 0x10 )
  {
    v_data1_ok = 0x01;
    // Convert the data to actual temperature
    // because the result is a 16 bit signed integer, it should
    // be stored to an "int16_t" type, which is always 16 bits
    // even when compiled on a 32 bit processor.
    t1_raw = (data1[1] << 8) | data1[0];
  }
}
// --------------------------------------------------------------
// --------------------------------------------------------------
void convert_t2()
{
  v_data2_ok = 0x00;
  if ( data2[7] == 0x10 )
  {
    v_data2_ok = 0x01;
    // Convert the data to actual temperature
    // because the result is a 16 bit signed integer, it should
    // be stored to an "int16_t" type, which is always 16 bits
    // even when compiled on a 32 bit processor.
    t2_raw = (data2[1] << 8) | data2[0];
  }
}
// --------------------------------------------------------------
// --------------------------------------------------------------
void meassure_and_read_X(byte i_style)
{
  ds18b20_1.reset();
  ds18b20_1.select(ROM_ID1);
  ds18b20_1.write(0x44, 1);        // start conversion, with parasite power on at the end

  ds18b20_2.reset();
  ds18b20_2.select(ROM_ID2);
  ds18b20_2.write(0x44, 1);        // start conversion, with parasite power on at the end

  if ( i_style == 1 )
  {
    alive_sign();
    delay(750);
  }
  else delay(750);

  ds18b20_1.reset();
  ds18b20_1.select(ROM_ID1);
  ds18b20_1.write(0xBE);
  for ( i = 0; i < 9; i++) data1[i] = ds18b20_1.read();
  ds18b20_1.reset();

  ds18b20_2.reset();
  ds18b20_2.select(ROM_ID2);
  ds18b20_2.write(0xBE);
  for ( i = 0; i < 9; i++) data2[i] = ds18b20_2.read();
  ds18b20_2.reset();
}
// --------------------------------------------------------------
// --------------------------------------------------------------
void lcd_print_temp_duo()
{
  lcd.setCursor(0, 0);
  lcd.print("In :     ");
  if ( v_data1_ok == 0x01 )
  {
    lcd.setCursor(7, 0);
    lcd_print_spaces(t1_in, 1);
    lcd.print(char(223));
    lcd.print("C  ");
  }
  else if ( data1[7] = 0xFF )
  {
    lcd.setCursor(7, 0);
    lcd.print("No Sensor");
  }
  else
  {
    lcd.setCursor(7, 0);
    lcd.print("Bad Sensor");
  }
  // --------------------------------------------------------------
  lcd.setCursor(0, 1);
  lcd.print("Out:     ");
  if ( v_data2_ok == 0x01 )
  {
    lcd.setCursor(7, 1);
    lcd_print_spaces(t2_out, 1);
    lcd.print(char(223));
    lcd.print("C  ");
  }
  else if ( data2[7] = 0xFF )
  {
    lcd.setCursor(7, 1);
    lcd.print("No Sensor ");
  }
  else
  {
    lcd.setCursor(7, 1);
    lcd.print("Bad Sensor");
  }
}
// --------------------------------------------------------------
// --------------------------------------------------------------
void lcd_print_spaces(float i_value, byte w_dec)
{
  if ( i_value >=  0 )
  {
    lcd.print(" ");         //instead of "-" for negative values
    if ( i_value <  10 ) lcd.print(" ");         // 10
    if ( i_value < 100 ) lcd.print(" ");         //100
  }
  else
  {
    lcd.print(" ");
    if ( i_value > -10 ) lcd.print(" ");         // 10
  }
  lcd.print(i_value, w_dec);
}
// --------------------------------------------------------------
// --------------------------------------------------------------
void alive_sign(void)
{
  if(v_index == 0) {lcd.setCursor(5,0); lcd.print("^");}
  if(v_index == 1) {lcd.setCursor(5,0); lcd.print("v");}
  if(v_index == 2) {lcd.setCursor(5,1); lcd.print("^");}
  if(v_index == 3) {lcd.setCursor(5,1); lcd.print("v");}
  v_index++;
  if ( v_index >= 4 ) v_index = 0;
}
// --------------------------------------------------------------
// --------------------------------------------------------------
